@@ -11,6 +11,7 @@ GQuark  bd_btrfs_error_quark (void) {
  */
 /**
  * bd_btrfs_device_info_copy: (skip)
+ * @info: (nullable): %BDBtrfsDeviceInfo to copy
  *
  * Creates a new copy of @info.
  */
@@ -30,6 +31,7 @@ BDBtrfsDeviceInfo* bd_btrfs_device_info_copy (BDBtrfsDeviceInfo *info) {
 
 /**
  * bd_btrfs_device_info_free: (skip)
+ * @info: (nullable): %BDBtrfsDeviceInfo to free
  *
  * Frees @info.
  */
@@ -61,6 +63,7 @@ GType  bd_btrfs_device_info_get_type () {
  */
 /**
  * bd_btrfs_subvolume_info_copy: (skip)
+ * @info: (nullable): %BDBtrfsSubvolumeInfo to copy
  *
  * Creates a new copy of @info.
  */
@@ -79,6 +82,7 @@ BDBtrfsSubvolumeInfo* bd_btrfs_subvolume_info_copy (BDBtrfsSubvolumeInfo *info)
 
 /**
  * bd_btrfs_subvolume_info_free: (skip)
+ * @info: (nullable): %BDBtrfsSubvolumeInfo to free
  *
  * Frees @info.
  */
@@ -111,6 +115,7 @@ GType  bd_btrfs_subvolume_info_get_type () {
  */
 /**
  * bd_btrfs_filesystem_info_copy: (skip)
+ * @info: (nullable): %BDBtrfsFilesystemInfo to copy
  *
  * Creates a new copy of @info.
  */
@@ -130,6 +135,7 @@ BDBtrfsFilesystemInfo* bd_btrfs_filesystem_info_copy (BDBtrfsFilesystemInfo *inf
 
 /**
  * bd_btrfs_filesystem_info_free: (skip)
+ * @info: (nullable): %BDBtrfsFilesystemInfo to free
  *
  * Frees @info.
  */
@@ -154,36 +160,6 @@ GType  bd_btrfs_filesystem_info_get_type () {
     return type;
 }
 
-static const gchar* const btrfs_functions[] = {
-    "bd_btrfs_is_tech_avail",
-    "bd_btrfs_create_volume",
-    "bd_btrfs_add_device",
-    "bd_btrfs_remove_device",
-    "bd_btrfs_create_subvolume",
-    "bd_btrfs_delete_subvolume",
-    "bd_btrfs_get_default_subvolume_id",
-    "bd_btrfs_set_default_subvolume",
-    "bd_btrfs_create_snapshot",
-    "bd_btrfs_list_devices",
-    "bd_btrfs_list_subvolumes",
-    "bd_btrfs_filesystem_info",
-    "bd_btrfs_mkfs",
-    "bd_btrfs_resize",
-    "bd_btrfs_check",
-    "bd_btrfs_repair",
-    "bd_btrfs_change_label",
-    NULL};
-
-const gchar* const* get_btrfs_functions (void) {
-    return btrfs_functions;
-}
-
-static const guint8 btrfs_num_functions = 17;
-
-guint8 get_btrfs_num_functions (void) {
-    return btrfs_num_functions;
-}
-
 static gboolean  bd_btrfs_is_tech_avail_stub (BDBtrfsTech tech, guint64 mode, GError **error) {
     g_critical ("The function 'bd_btrfs_is_tech_avail' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
@@ -197,9 +173,9 @@ static gboolean  (*_bd_btrfs_is_tech_avail) (BDBtrfsTech tech, guint64 mode, GEr
  * bd_btrfs_is_tech_avail:
  * @tech: the queried tech
  * @mode: a bit mask of queried modes of operation (#BDBtrfsTechMode) for @tech
- * @error: (out): place to store error (details about why the @tech-@mode combination is not available)
+ * @error: (out) (optional): place to store error (details about why the @tech-@mode combination is not available)
  *
- * Returns: whether the @tech-@mode combination is avaible -- supported by the
+ * Returns: whether the @tech-@mode combination is available -- supported by the
  *          plugin implementation and having all the runtime dependencies available
  */
 gboolean  bd_btrfs_is_tech_avail (BDBtrfsTech tech, guint64 mode, GError **error) {
@@ -219,12 +195,12 @@ static gboolean  (*_bd_btrfs_create_volume) (const gchar **devices, const gchar
 /**
  * bd_btrfs_create_volume:
  * @devices: (array zero-terminated=1): list of devices to create btrfs volume from
- * @label: (allow-none): label for the volume
- * @data_level: (allow-none): RAID level for the data or %NULL to use the default
- * @md_level: (allow-none): RAID level for the metadata or %NULL to use the default
- * @extra: (allow-none) (array zero-terminated=1): extra options for the volume creation (right now
+ * @label: (nullable): label for the volume
+ * @data_level: (nullable): RAID level for the data or %NULL to use the default
+ * @md_level: (nullable): RAID level for the metadata or %NULL to use the default
+ * @extra: (nullable) (array zero-terminated=1): extra options for the volume creation (right now
  *                                                 passed to the 'mkfs.btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the new btrfs volume was created from @devices or not
  *
@@ -250,9 +226,9 @@ static gboolean  (*_bd_btrfs_add_device) (const gchar *mountpoint, const gchar *
  * bd_btrfs_add_device:
  * @mountpoint: mountpoint of the btrfs volume to add new device to
  * @device: a device to add to the btrfs volume
- * @extra: (allow-none) (array zero-terminated=1): extra options for the addition (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the addition (right now
  *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @device was successfully added to the @mountpoint btrfs volume or not
  *
@@ -276,9 +252,9 @@ static gboolean  (*_bd_btrfs_remove_device) (const gchar *mountpoint, const gcha
  * bd_btrfs_remove_device:
  * @mountpoint: mountpoint of the btrfs volume to remove device from
  * @device: a device to remove from the btrfs volume
- * @extra: (allow-none) (array zero-terminated=1): extra options for the removal (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the removal (right now
  *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @device was successfully removed from the @mountpoint btrfs volume or not
  *
@@ -302,9 +278,9 @@ static gboolean  (*_bd_btrfs_create_subvolume) (const gchar *mountpoint, const g
  * bd_btrfs_create_subvolume:
  * @mountpoint: mountpoint of the btrfs volume to create subvolume under
  * @name: name of the subvolume
- * @extra: (allow-none) (array zero-terminated=1): extra options for the subvolume creation (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the subvolume creation (right now
  *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @mountpoint/@name subvolume was successfully created or not
  *
@@ -328,9 +304,9 @@ static gboolean  (*_bd_btrfs_delete_subvolume) (const gchar *mountpoint, const g
  * bd_btrfs_delete_subvolume:
  * @mountpoint: mountpoint of the btrfs volume to delete subvolume from
  * @name: name of the subvolume
- * @extra: (allow-none) (array zero-terminated=1): extra options for the subvolume deletion (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the subvolume deletion (right now
  *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @mountpoint/@name subvolume was successfully deleted or not
  *
@@ -353,10 +329,10 @@ static guint64  (*_bd_btrfs_get_default_subvolume_id) (const gchar *mountpoint,
 /**
  * bd_btrfs_get_default_subvolume_id:
  * @mountpoint: mountpoint of the volume to get the default subvolume ID of
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: ID of the @mountpoint volume's default subvolume. If 0,
- * @error) may be set to indicate error
+ *          @error may be set to indicate error
  *
  * Tech category: %BD_BTRFS_TECH_SUBVOL-%BD_BTRFS_TECH_MODE_QUERY
  */
@@ -378,12 +354,12 @@ static gboolean  (*_bd_btrfs_set_default_subvolume) (const gchar *mountpoint, gu
  * bd_btrfs_set_default_subvolume:
  * @mountpoint: mountpoint of the volume to set the default subvolume ID of
  * @subvol_id: ID of the subvolume to be set as the default subvolume
- * @extra: (allow-none) (array zero-terminated=1): extra options for the setting (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the setting (right now
  *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @mountpoint volume's default subvolume was correctly set
- * to @subvol_id or not
+ *          to @subvol_id or not
  *
  * Tech category: %BD_BTRFS_TECH_SUBVOL-%BD_BTRFS_TECH_MODE_MODIFY
  */
@@ -406,9 +382,9 @@ static gboolean  (*_bd_btrfs_create_snapshot) (const gchar *source, const gchar
  * @source: path to source subvolume
  * @dest: path to new snapshot volume
  * @ro: whether the snapshot should be read-only
- * @extra: (allow-none) (array zero-terminated=1): extra options for the snapshot creation (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the snapshot creation (right now
  *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @dest snapshot of @source was successfully created or not
  *
@@ -431,7 +407,7 @@ static BDBtrfsDeviceInfo** (*_bd_btrfs_list_devices) (const gchar *device, GErro
 /**
  * bd_btrfs_list_devices:
  * @device: a device that is part of the queried btrfs volume
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: (array zero-terminated=1): information about the devices that are part of the btrfs volume
  * containing @device or %NULL in case of error
@@ -456,7 +432,7 @@ static BDBtrfsSubvolumeInfo** (*_bd_btrfs_list_subvolumes) (const gchar *mountpo
  * bd_btrfs_list_subvolumes:
  * @mountpoint: a mountpoint of the queried btrfs volume
  * @snapshots_only: whether to list only snapshot subvolumes or not
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: (array zero-terminated=1): information about the subvolumes that are part of the btrfs volume
  * mounted at @mountpoint or %NULL in case of error
@@ -483,7 +459,7 @@ static BDBtrfsFilesystemInfo* (*_bd_btrfs_filesystem_info) (const gchar *device,
 /**
  * bd_btrfs_filesystem_info:
  * @device: a device that is part of the queried btrfs volume
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: information about the @device's volume's filesystem or %NULL in case of error
  *
@@ -506,12 +482,12 @@ static gboolean  (*_bd_btrfs_mkfs) (const gchar **devices, const gchar *label, c
 /**
  * bd_btrfs_mkfs:
  * @devices: (array zero-terminated=1): list of devices to create btrfs volume from
- * @label: (allow-none): label for the volume
- * @data_level: (allow-none): RAID level for the data or %NULL to use the default
- * @md_level: (allow-none): RAID level for the metadata or %NULL to use the default
- * @extra: (allow-none) (array zero-terminated=1): extra options for the volume creation (right now
+ * @label: (nullable): label for the volume
+ * @data_level: (nullable): RAID level for the data or %NULL to use the default
+ * @md_level: (nullable): RAID level for the metadata or %NULL to use the default
+ * @extra: (nullable) (array zero-terminated=1): extra options for the volume creation (right now
  *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the new btrfs volume was created from @devices or not
  *
@@ -537,9 +513,9 @@ static gboolean  (*_bd_btrfs_resize) (const gchar *mountpoint, guint64 size, con
  * bd_btrfs_resize:
  * @mountpoint: a mountpoint of the to be resized btrfs filesystem
  * @size: requested new size
- * @extra: (allow-none) (array zero-terminated=1): extra options for the volume resize (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the volume resize (right now
  *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @mountpoint filesystem was successfully resized to @size
  * or not
@@ -563,9 +539,9 @@ static gboolean  (*_bd_btrfs_check) (const gchar *device, const BDExtraArg **ext
 /**
  * bd_btrfs_check:
  * @device: a device that is part of the checked btrfs volume
- * @extra: (allow-none) (array zero-terminated=1): extra options for the check (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the check (right now
  *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the filesystem was successfully checked or not
  *
@@ -588,9 +564,9 @@ static gboolean  (*_bd_btrfs_repair) (const gchar *device, const BDExtraArg **ex
 /**
  * bd_btrfs_repair:
  * @device: a device that is part of the to be repaired btrfs volume
- * @extra: (allow-none) (array zero-terminated=1): extra options for the repair (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the repair (right now
  *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the filesystem was successfully checked and repaired or not
  *
@@ -601,152 +577,142 @@ gboolean  bd_btrfs_repair (const gchar *device, const BDExtraArg **extra, GError
 }
 
 
-static gboolean  bd_btrfs_change_label_stub (const gchar *mountpoint, const gchar *label, const BDExtraArg **extra, GError **error) {
+static gboolean  bd_btrfs_change_label_stub (const gchar *mountpoint, const gchar *label, GError **error) {
     g_critical ("The function 'bd_btrfs_change_label' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_btrfs_change_label' called, but not implemented!");
     return FALSE;
 }
 
-static gboolean  (*_bd_btrfs_change_label) (const gchar *mountpoint, const gchar *label, const BDExtraArg **extra, GError **error) = bd_btrfs_change_label_stub;
+static gboolean  (*_bd_btrfs_change_label) (const gchar *mountpoint, const gchar *label, GError **error) = bd_btrfs_change_label_stub;
 
 /**
  * bd_btrfs_change_label:
  * @mountpoint: a mountpoint of the btrfs filesystem to change label of
  * @label: new label for the filesystem
- * @extra: (allow-none) (array zero-terminated=1): extra options for the volume creation (right now
- *                                                 passed to the 'btrfs' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the label of the @mountpoint filesystem was successfully set
  * to @label or not
  *
  * Tech category: %BD_BTRFS_TECH_FS-%BD_BTRFS_TECH_MODE_MODIFY
  */
-gboolean  bd_btrfs_change_label (const gchar *mountpoint, const gchar *label, const BDExtraArg **extra, GError **error) {
-    return _bd_btrfs_change_label (mountpoint, label, extra, error);
+gboolean  bd_btrfs_change_label (const gchar *mountpoint, const gchar *label, GError **error) {
+    return _bd_btrfs_change_label (mountpoint, label, error);
 }
 
 
-gpointer load_btrfs_from_plugin(const gchar *so_name) {
+static gpointer load_btrfs_from_plugin(const gchar *so_name) {
     void *handle = NULL;
     char *error = NULL;
-    gboolean (*check_fn) (void) = NULL;
     gboolean (*init_fn) (void) = NULL;
 
     handle = dlopen(so_name, RTLD_LAZY);
     if (!handle) {
-        g_warning("failed to load module btrfs: %s", dlerror());
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load module btrfs: %s", dlerror());
         return NULL;
     }
 
-    dlerror();
-    * (void**) (&check_fn) = dlsym(handle, "bd_btrfs_check_deps");
-    if ((error = dlerror()) != NULL)
-        g_debug("failed to load the check() function for btrfs: %s", error);
-    if (!g_getenv ("LIBBLOCKDEV_SKIP_DEP_CHECKS") && check_fn && !check_fn()) {
-        dlclose(handle);
-        return NULL;
-    }    check_fn = NULL;
-
     dlerror();
     * (void**) (&init_fn) = dlsym(handle, "bd_btrfs_init");
     if ((error = dlerror()) != NULL)
-        g_debug("failed to load the init() function for btrfs: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_DEBUG, "failed to load the init() function for btrfs: %s", error);
+    /* coverity[dead_error_condition] */
     if (init_fn && !init_fn()) {
         dlclose(handle);
         return NULL;
-    }    init_fn = NULL;
+    }
+    init_fn = NULL;
 
     dlerror();
     * (void**) (&_bd_btrfs_is_tech_avail) = dlsym(handle, "bd_btrfs_is_tech_avail");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_is_tech_avail: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_is_tech_avail: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_create_volume) = dlsym(handle, "bd_btrfs_create_volume");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_create_volume: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_create_volume: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_add_device) = dlsym(handle, "bd_btrfs_add_device");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_add_device: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_add_device: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_remove_device) = dlsym(handle, "bd_btrfs_remove_device");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_remove_device: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_remove_device: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_create_subvolume) = dlsym(handle, "bd_btrfs_create_subvolume");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_create_subvolume: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_create_subvolume: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_delete_subvolume) = dlsym(handle, "bd_btrfs_delete_subvolume");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_delete_subvolume: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_delete_subvolume: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_get_default_subvolume_id) = dlsym(handle, "bd_btrfs_get_default_subvolume_id");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_get_default_subvolume_id: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_get_default_subvolume_id: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_set_default_subvolume) = dlsym(handle, "bd_btrfs_set_default_subvolume");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_set_default_subvolume: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_set_default_subvolume: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_create_snapshot) = dlsym(handle, "bd_btrfs_create_snapshot");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_create_snapshot: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_create_snapshot: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_list_devices) = dlsym(handle, "bd_btrfs_list_devices");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_list_devices: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_list_devices: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_list_subvolumes) = dlsym(handle, "bd_btrfs_list_subvolumes");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_list_subvolumes: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_list_subvolumes: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_filesystem_info) = dlsym(handle, "bd_btrfs_filesystem_info");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_filesystem_info: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_filesystem_info: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_mkfs) = dlsym(handle, "bd_btrfs_mkfs");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_mkfs: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_mkfs: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_resize) = dlsym(handle, "bd_btrfs_resize");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_resize: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_resize: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_check) = dlsym(handle, "bd_btrfs_check");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_check: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_check: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_repair) = dlsym(handle, "bd_btrfs_repair");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_repair: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_repair: %s", error);
 
     dlerror();
     * (void**) (&_bd_btrfs_change_label) = dlsym(handle, "bd_btrfs_change_label");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_btrfs_change_label: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_btrfs_change_label: %s", error);
 
     return handle;
 }
 
-gboolean unload_btrfs (gpointer handle) {
+static gboolean unload_btrfs (gpointer handle) {
     char *error = NULL;
     gboolean (*close_fn) (void) = NULL;
 
@@ -771,7 +737,8 @@ gboolean unload_btrfs (gpointer handle) {
     dlerror();
     * (void**) (&close_fn) = dlsym(handle, "bd_btrfs_close");
     if (((error = dlerror()) != NULL) || !close_fn)
-        g_debug("failed to load the close_plugin() function for btrfs: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_DEBUG, "failed to load the close_plugin() function for btrfs: %s", error);
+    /* coverity[dead_error_condition] */
     if (close_fn) {
         close_fn();
     }
