@@ -18,6 +18,7 @@ GQuark  bd_md_error_quark (void) {
  */
 /**
  * bd_md_examine_data_copy: (skip)
+ * @data: (nullable): %BDMDExamineData to copy
  *
  * Creates a new copy of @data.
  */
@@ -43,6 +44,7 @@ BDMDExamineData* bd_md_examine_data_copy (BDMDExamineData *data) {
 
 /**
  * bd_md_examine_data_free: (skip)
+ * @data: (nullable): %BDMDExamineData to free
  *
  * Frees @data.
  */
@@ -88,9 +90,11 @@ GType  bd_md_examine_data_get_type () {
  * @spare_devices: number of spare devices in the MD array
  * @clean: whether the MD array is clean or not
  * @uuid: uuid of the MD array
+ * @container: path of the MD container this device belongs to
  */
 /**
  * bd_md_detail_data_copy: (skip)
+ * @data: (nullable): %BDMDDetailData to copy
  *
  * Creates a new copy of @data.
  */
@@ -114,12 +118,14 @@ BDMDDetailData* bd_md_detail_data_copy (BDMDDetailData *data) {
     new_data->spare_devices = data->spare_devices;
     new_data->clean = data->clean;
     new_data->uuid = g_strdup (data->uuid);
+    new_data->container = g_strdup (data->container);
 
     return new_data;
 }
 
 /**
  * bd_md_detail_data_free: (skip)
+ * @data: (nullable): %BDMDDetailData to free
  *
  * Frees @data.
  */
@@ -133,6 +139,7 @@ void  bd_md_detail_data_free (BDMDDetailData *data) {
     g_free (data->creation_time);
     g_free (data->level);
     g_free (data->uuid);
+    g_free (data->container);
 
     g_free (data);
 }
@@ -149,40 +156,6 @@ GType  bd_md_detail_data_get_type () {
     return type;
 }
 
-static const gchar* const mdraid_functions[] = {
-    "bd_md_is_tech_avail",
-    "bd_md_get_superblock_size",
-    "bd_md_create",
-    "bd_md_destroy",
-    "bd_md_deactivate",
-    "bd_md_activate",
-    "bd_md_run",
-    "bd_md_nominate",
-    "bd_md_denominate",
-    "bd_md_add",
-    "bd_md_remove",
-    "bd_md_examine",
-    "bd_md_detail",
-    "bd_md_canonicalize_uuid",
-    "bd_md_get_md_uuid",
-    "bd_md_node_from_name",
-    "bd_md_name_from_node",
-    "bd_md_get_status",
-    "bd_md_set_bitmap_location",
-    "bd_md_get_bitmap_location",
-    "bd_md_request_sync_action",
-    NULL};
-
-const gchar* const* get_mdraid_functions (void) {
-    return mdraid_functions;
-}
-
-static const guint8 mdraid_num_functions = 21;
-
-guint8 get_mdraid_num_functions (void) {
-    return mdraid_num_functions;
-}
-
 static gboolean  bd_md_is_tech_avail_stub (BDMDTech tech, guint64 mode, GError **error) {
     g_critical ("The function 'bd_md_is_tech_avail' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
@@ -196,7 +169,7 @@ static gboolean  (*_bd_md_is_tech_avail) (BDMDTech tech, guint64 mode, GError **
  * bd_md_is_tech_avail:
  * @tech: the queried tech
  * @mode: a bit mask of queried modes of operation for @tech
- * @error: (out): place to store error (details about why the @tech-@mode combination is not available)
+ * @error: (out) (optional): place to store error (details about why the @tech-@mode combination is not available)
  *
  * Returns: whether the @tech-@mode combination is available -- supported by the
  *          plugin implementation and having all the runtime dependencies available
@@ -218,8 +191,8 @@ static guint64  (*_bd_md_get_superblock_size) (guint64 member_size, const gchar
 /**
  * bd_md_get_superblock_size:
  * @member_size: size of an array member
- * @version: (allow-none): metadata version or %NULL to use the current default version
- * @error: (out): place to store error (if any)
+ * @version: (nullable): metadata version or %NULL to use the current default version
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: Calculated superblock size for an array with a given @member_size
  * and metadata @version or default if unsupported @version is used.
@@ -231,14 +204,14 @@ guint64  bd_md_get_superblock_size (guint64 member_size, const gchar *version, G
 }
 
 
-static gboolean  bd_md_create_stub (const gchar *device_name, const gchar *level, const gchar **disks, guint64 spares, const gchar *version, gboolean bitmap, guint64 chunk_size, const BDExtraArg **extra, GError **error) {
+static gboolean  bd_md_create_stub (const gchar *device_name, const gchar *level, const gchar **disks, guint64 spares, const gchar *version, const gchar *bitmap, guint64 chunk_size, const BDExtraArg **extra, GError **error) {
     g_critical ("The function 'bd_md_create' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_md_create' called, but not implemented!");
     return FALSE;
 }
 
-static gboolean  (*_bd_md_create) (const gchar *device_name, const gchar *level, const gchar **disks, guint64 spares, const gchar *version, gboolean bitmap, guint64 chunk_size, const BDExtraArg **extra, GError **error) = bd_md_create_stub;
+static gboolean  (*_bd_md_create) (const gchar *device_name, const gchar *level, const gchar **disks, guint64 spares, const gchar *version, const gchar *bitmap, guint64 chunk_size, const BDExtraArg **extra, GError **error) = bd_md_create_stub;
 
 /**
  * bd_md_create:
@@ -246,18 +219,18 @@ static gboolean  (*_bd_md_create) (const gchar *device_name, const gchar *level,
  * @level: RAID level (as understood by mdadm, see mdadm(8))
  * @disks: (array zero-terminated=1): disks to use for the new RAID (including spares)
  * @spares: number of spare devices
- * @version: (allow-none): metadata version
- * @bitmap: whether to create an internal bitmap on the device or not
+ * @version: (nullable): metadata version
+ * @bitmap: (nullable): write-intent bitmap location ('none', 'internal') or %NULL to let mdadm decide (i.e. internal > 100GB)
  * @chunk_size: chunk size of the device to create
- * @extra: (allow-none) (array zero-terminated=1): extra options for the creation (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the creation (right now
  *                                                 passed to the 'mdadm' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the new MD RAID device @device_name was successfully created or not
  *
  * Tech category: %BD_MD_TECH_MDRAID-%BD_MD_TECH_MODE_CREATE
  */
-gboolean  bd_md_create (const gchar *device_name, const gchar *level, const gchar **disks, guint64 spares, const gchar *version, gboolean bitmap, guint64 chunk_size, const BDExtraArg **extra, GError **error) {
+gboolean  bd_md_create (const gchar *device_name, const gchar *level, const gchar **disks, guint64 spares, const gchar *version, const gchar *bitmap, guint64 chunk_size, const BDExtraArg **extra, GError **error) {
     return _bd_md_create (device_name, level, disks, spares, version, bitmap, chunk_size, extra, error);
 }
 
@@ -274,7 +247,7 @@ static gboolean  (*_bd_md_destroy) (const gchar *device, GError **error) = bd_md
 /**
  * bd_md_destroy:
  * @device: device to destroy MD RAID metadata on
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the MD RAID metadata was successfully destroyed on @device or not
  *
@@ -297,7 +270,7 @@ static gboolean  (*_bd_md_deactivate) (const gchar *raid_spec, GError **error) =
 /**
  * bd_md_deactivate:
  * @raid_spec: specification of the RAID device (name, node or path)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the RAID device @raid_spec was successfully deactivated or not
  *
@@ -319,13 +292,13 @@ static gboolean  (*_bd_md_activate) (const gchar *raid_spec, const gchar **membe
 
 /**
  * bd_md_activate:
- * @raid_spec: (allow-none): specification of the RAID device (name, node or path) to activate (if not given "--scan" is implied and @members is ignored)
- * @members: (allow-none) (array zero-terminated=1): member devices to be considered for @device activation
- * @uuid: (allow-none): UUID (in the MD RAID format!) of the MD RAID to activate
+ * @raid_spec: (nullable): specification of the RAID device (name, node or path) to activate (if not given "--scan" is implied and @members is ignored)
+ * @members: (nullable) (array zero-terminated=1): member devices to be considered for @device activation
+ * @uuid: (nullable): UUID (in the MD RAID format!) of the MD RAID to activate
  * @start_degraded: whether to start the array even if it's degraded
- * @extra: (allow-none) (array zero-terminated=1): extra options for the activation (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the activation (right now
  *                                                 passed to the 'mdadm' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the MD RAID @device was successfully activated or not
  *
@@ -350,7 +323,7 @@ static gboolean  (*_bd_md_run) (const gchar *raid_spec, GError **error) = bd_md_
 /**
  * bd_md_run:
  * @raid_spec: specification of the (possibly degraded) RAID device (name, node or path) to be started
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @raid_spec was successfully started or not
  *
@@ -373,7 +346,7 @@ static gboolean  (*_bd_md_nominate) (const gchar *device, GError **error) = bd_m
 /**
  * bd_md_nominate:
  * @device: device to nominate (add to its appropriate RAID) as a MD RAID device
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @device was successfully nominated (added to its
  * appropriate RAID) or not
@@ -399,7 +372,7 @@ static gboolean  (*_bd_md_denominate) (const gchar *device, GError **error) = bd
 /**
  * bd_md_denominate:
  * @device: device to denominate (remove from its appropriate RAID) as a MD RAID device
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @device was successfully denominated (added to its
  * appropriate RAID) or not
@@ -428,9 +401,9 @@ static gboolean  (*_bd_md_add) (const gchar *raid_spec, const gchar *device, gui
  * @device: name of the device to add to the @raid_spec RAID device
  * @raid_devs: number of devices the @raid_spec RAID should actively use or 0
  *             to leave unspecified (see below)
- * @extra: (allow-none) (array zero-terminated=1): extra options for the addition (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the addition (right now
  *                                                 passed to the 'mdadm' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @device was successfully added to the @raid_spec RAID or
  * not
@@ -463,9 +436,9 @@ static gboolean  (*_bd_md_remove) (const gchar *raid_spec, const gchar *device,
  * @raid_spec: specification of the RAID device (name, node or path) to remove @device from
  * @device: device to remove from the @raid_spec RAID
  * @fail: whether to mark the @device as failed before removing
- * @extra: (allow-none) (array zero-terminated=1): extra options for the removal (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the removal (right now
  *                                                 passed to the 'mdadm' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @device was successfully removed from the @raid_spec
  * RAID or not.
@@ -489,7 +462,7 @@ static BDMDExamineData* (*_bd_md_examine) (const gchar *device, GError **error)
 /**
  * bd_md_examine:
  * @device: name of the device (a member of an MD RAID) to examine
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: information about the MD RAID extracted from the @device
  *
@@ -512,7 +485,7 @@ static BDMDDetailData* (*_bd_md_detail) (const gchar *raid_spec, GError **error)
 /**
  * bd_md_detail:
  * @raid_spec: specification of the RAID device (name, node or path) to examine
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: information about the MD RAID @raid_spec
  *
@@ -535,9 +508,9 @@ static gchar* (*_bd_md_canonicalize_uuid) (const gchar *uuid, GError **error) =
 /**
  * bd_md_canonicalize_uuid:
  * @uuid: UUID to canonicalize
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
- * Returns: (transfer full): cannonicalized form of @uuid
+ * Returns: (transfer full): canonicalized form of @uuid
  *
  * This function expects a UUID in the form that mdadm returns. The change is as
  * follows: 3386ff85:f5012621:4a435f06:1eb47236 -> 3386ff85-f501-2621-4a43-5f061eb47236
@@ -561,7 +534,7 @@ static gchar* (*_bd_md_get_md_uuid) (const gchar *uuid, GError **error) = bd_md_
 /**
  * bd_md_get_md_uuid:
  * @uuid: UUID to transform into format used by MD RAID
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: (transfer full): transformed form of @uuid
  *
@@ -589,7 +562,7 @@ static gchar* (*_bd_md_node_from_name) (const gchar *name, GError **error) = bd_
 /**
  * bd_md_node_from_name:
  * @name: name of the MD RAID
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: device node of the @name MD RAID or %NULL in case of error
  *
@@ -612,7 +585,7 @@ static gchar* (*_bd_md_name_from_node) (const gchar *node, GError **error) = bd_
 /**
  * bd_md_name_from_node:
  * @node: path of the MD RAID's device node
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: @name of the MD RAID the device node belongs to or %NULL in case of error
  *
@@ -635,7 +608,7 @@ static gchar* (*_bd_md_get_status) (const gchar *raid_spec, GError **error) = bd
 /**
  * bd_md_get_status
  * @raid_spec: specification of the RAID device (name, node or path) to get status
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: (transfer full): status of the @raid_spec RAID.
  *
@@ -659,7 +632,7 @@ static gboolean  (*_bd_md_set_bitmap_location) (const gchar *raid_spec, const gc
  * bd_md_set_bitmap_location:
  * @raid_spec: specification of the RAID device (name, node or path) to set the bitmap location
  * @location: bitmap location (none, internal or path)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether @location was successfully set for @raid_spec
  *
@@ -682,7 +655,7 @@ static gchar* (*_bd_md_get_bitmap_location) (const gchar *raid_spec, GError **er
 /**
  * bd_md_get_bitmap_location:
  * @raid_spec: specification of the RAID device (name, node or path) to get the bitmap location
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: (transfer full): bitmap location for @raid_spec
  *
@@ -706,7 +679,7 @@ static gboolean  (*_bd_md_request_sync_action) (const gchar *raid_spec, const gc
  * bd_md_request_sync_action:
  * @raid_spec: specification of the RAID device (name, node or path) to request sync action on
  * @action: requested sync action (resync, recovery, check, repair or idle)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the @action was successfully requested for the @raid_spec
  * RAID or not.
@@ -718,145 +691,137 @@ gboolean  bd_md_request_sync_action (const gchar *raid_spec, const gchar *action
 }
 
 
-gpointer load_mdraid_from_plugin(const gchar *so_name) {
+static gpointer load_mdraid_from_plugin(const gchar *so_name) {
     void *handle = NULL;
     char *error = NULL;
-    gboolean (*check_fn) (void) = NULL;
     gboolean (*init_fn) (void) = NULL;
 
     handle = dlopen(so_name, RTLD_LAZY);
     if (!handle) {
-        g_warning("failed to load module mdraid: %s", dlerror());
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load module mdraid: %s", dlerror());
         return NULL;
     }
 
-    dlerror();
-    * (void**) (&check_fn) = dlsym(handle, "bd_md_check_deps");
-    if ((error = dlerror()) != NULL)
-        g_debug("failed to load the check() function for mdraid: %s", error);
-    if (!g_getenv ("LIBBLOCKDEV_SKIP_DEP_CHECKS") && check_fn && !check_fn()) {
-        dlclose(handle);
-        return NULL;
-    }    check_fn = NULL;
-
     dlerror();
     * (void**) (&init_fn) = dlsym(handle, "bd_md_init");
     if ((error = dlerror()) != NULL)
-        g_debug("failed to load the init() function for mdraid: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_DEBUG, "failed to load the init() function for mdraid: %s", error);
+    /* coverity[dead_error_condition] */
     if (init_fn && !init_fn()) {
         dlclose(handle);
         return NULL;
-    }    init_fn = NULL;
+    }
+    init_fn = NULL;
 
     dlerror();
     * (void**) (&_bd_md_is_tech_avail) = dlsym(handle, "bd_md_is_tech_avail");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_is_tech_avail: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_is_tech_avail: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_get_superblock_size) = dlsym(handle, "bd_md_get_superblock_size");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_get_superblock_size: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_get_superblock_size: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_create) = dlsym(handle, "bd_md_create");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_create: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_create: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_destroy) = dlsym(handle, "bd_md_destroy");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_destroy: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_destroy: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_deactivate) = dlsym(handle, "bd_md_deactivate");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_deactivate: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_deactivate: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_activate) = dlsym(handle, "bd_md_activate");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_activate: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_activate: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_run) = dlsym(handle, "bd_md_run");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_run: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_run: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_nominate) = dlsym(handle, "bd_md_nominate");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_nominate: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_nominate: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_denominate) = dlsym(handle, "bd_md_denominate");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_denominate: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_denominate: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_add) = dlsym(handle, "bd_md_add");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_add: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_add: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_remove) = dlsym(handle, "bd_md_remove");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_remove: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_remove: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_examine) = dlsym(handle, "bd_md_examine");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_examine: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_examine: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_detail) = dlsym(handle, "bd_md_detail");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_detail: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_detail: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_canonicalize_uuid) = dlsym(handle, "bd_md_canonicalize_uuid");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_canonicalize_uuid: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_canonicalize_uuid: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_get_md_uuid) = dlsym(handle, "bd_md_get_md_uuid");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_get_md_uuid: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_get_md_uuid: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_node_from_name) = dlsym(handle, "bd_md_node_from_name");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_node_from_name: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_node_from_name: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_name_from_node) = dlsym(handle, "bd_md_name_from_node");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_name_from_node: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_name_from_node: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_get_status) = dlsym(handle, "bd_md_get_status");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_get_status: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_get_status: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_set_bitmap_location) = dlsym(handle, "bd_md_set_bitmap_location");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_set_bitmap_location: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_set_bitmap_location: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_get_bitmap_location) = dlsym(handle, "bd_md_get_bitmap_location");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_get_bitmap_location: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_get_bitmap_location: %s", error);
 
     dlerror();
     * (void**) (&_bd_md_request_sync_action) = dlsym(handle, "bd_md_request_sync_action");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_md_request_sync_action: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_md_request_sync_action: %s", error);
 
     return handle;
 }
 
-gboolean unload_mdraid (gpointer handle) {
+static gboolean unload_mdraid (gpointer handle) {
     char *error = NULL;
     gboolean (*close_fn) (void) = NULL;
 
@@ -885,7 +850,8 @@ gboolean unload_mdraid (gpointer handle) {
     dlerror();
     * (void**) (&close_fn) = dlsym(handle, "bd_md_close");
     if (((error = dlerror()) != NULL) || !close_fn)
-        g_debug("failed to load the close_plugin() function for mdraid: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_DEBUG, "failed to load the close_plugin() function for mdraid: %s", error);
+    /* coverity[dead_error_condition] */
     if (close_fn) {
         close_fn();
     }
