@@ -2,31 +2,6 @@ GQuark  bd_s390_error_quark (void) {
         return g_quark_from_static_string ("g-bd-s390-error-quark");
 }
 
-static const gchar* const s390_functions[] = {
-    "bd_s390_is_tech_avail",
-    "bd_s390_dasd_format",
-    "bd_s390_dasd_needs_format",
-    "bd_s390_dasd_online",
-    "bd_s390_dasd_is_ldl",
-    "bd_s390_dasd_is_fba",
-    "bd_s390_sanitize_dev_input",
-    "bd_s390_zfcp_sanitize_wwpn_input",
-    "bd_s390_zfcp_sanitize_lun_input",
-    "bd_s390_zfcp_online",
-    "bd_s390_zfcp_scsi_offline",
-    "bd_s390_zfcp_offline",
-    NULL};
-
-const gchar* const* get_s390_functions (void) {
-    return s390_functions;
-}
-
-static const guint8 s390_num_functions = 12;
-
-guint8 get_s390_num_functions (void) {
-    return s390_num_functions;
-}
-
 static gboolean  bd_s390_is_tech_avail_stub (BDS390Tech tech, guint64 mode, GError **error) {
     g_critical ("The function 'bd_s390_is_tech_avail' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
@@ -40,7 +15,7 @@ static gboolean  (*_bd_s390_is_tech_avail) (BDS390Tech tech, guint64 mode, GErro
  * bd_s390_is_tech_avail:
  * @tech: the queried tech
  * @mode: a bit mask of queried modes of operation (#BDS390TechMode) for @tech
- * @error: (out): place to store error (details about why the @tech-@mode combination is not available)
+ * @error: (out) (optional): place to store error (details about why the @tech-@mode combination is not available)
  *
  * Returns: whether the @tech-@mode combination is available -- supported by the
  *          plugin implementation and having all the runtime dependencies available
@@ -62,9 +37,9 @@ static gboolean  (*_bd_s390_dasd_format) (const gchar *dasd, const BDExtraArg **
 /**
  * bd_s390_dasd_format:
  * @dasd: dasd to format
- * @extra: (allow-none) (array zero-terminated=1): extra options for the formatting (right now
+ * @extra: (nullable) (array zero-terminated=1): extra options for the formatting (right now
  *                                                 passed to the 'dasdfmt' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether dasdfmt was successful or not
  *
@@ -87,7 +62,7 @@ static gboolean  (*_bd_s390_dasd_needs_format) (const gchar *dasd, GError **erro
 /**
  * bd_s390_dasd_needs_format:
  * @dasd: dasd to check, whether it needs dasdfmt run on it
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether a dasd needs dasdfmt run against it
  *
@@ -110,7 +85,7 @@ static gboolean  (*_bd_s390_dasd_online) (const gchar *dasd, GError **error) = b
 /**
  * bd_s390_dasd_online:
  * @dasd: dasd to switch online
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether a dasd was successfully switched online
  *
@@ -133,7 +108,7 @@ static gboolean  (*_bd_s390_dasd_is_ldl) (const gchar *dasd, GError **error) = b
 /**
  * bd_s390_dasd_is_Ldl:
  * @dasd: dasd to check, whether it is LDL formatted
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether a dasd is LDL formatted
  *
@@ -156,7 +131,7 @@ static gboolean  (*_bd_s390_dasd_is_fba) (const gchar *dasd, GError **error) = b
 /**
  * bd_s390_dasd_is_fba:
  * @dasd: dasd to check, whether it is FBA
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether a dasd is FBA
  *
@@ -178,8 +153,8 @@ static gchar* (*_bd_s390_sanitize_dev_input) (const gchar *dev, GError **error)
 
 /**
  * bd_s390_sanitize_dev_input:
- * @dev a DASD or zFCP device number
- * @error: (out): place to store error (if any)
+ * @dev: a DASD or zFCP device number
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: (transfer full): a synthesized dasd or zfcp device number
  *
@@ -201,8 +176,8 @@ static gchar* (*_bd_s390_zfcp_sanitize_wwpn_input) (const gchar *wwpn, GError **
 
 /**
  * bd_s390_zfcp_sanitize_wwpn_input:
- * @wwpn a zFCP WWPN identifier
- * @error: (out): place to store error (if any)
+ * @wwpn: a zFCP WWPN identifier
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: (transfer full): a synthesized zFCP WWPN
  *
@@ -224,8 +199,8 @@ static gchar* (*_bd_s390_zfcp_sanitize_lun_input) (const gchar *lun, GError **er
 
 /**
  * bd_s390_zfcp_sanitize_lun_input:
- * @lun a zFCP LUN identifier
- * @error: (out): place to store error (if any)
+ * @lun: a zFCP LUN identifier
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: (transfer full): a synthesized zFCP LUN
  *
@@ -247,10 +222,10 @@ static gboolean  (*_bd_s390_zfcp_online) (const gchar *devno, const gchar *wwpn,
 
 /**
  * bd_s390_zfcp_online:
- * @devno a zFCP device number
- * @wwpn a zFCP WWPN identifier
- * @lun a zFCP LUN identifier
- * @error: (out): place to store error (if any)
+ * @devno: a zFCP device number
+ * @wwpn: a zFCP WWPN identifier
+ * @lun: a zFCP LUN identifier
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether a zFCP device was successfully switched online
  *
@@ -272,10 +247,10 @@ static gboolean  (*_bd_s390_zfcp_scsi_offline) (const gchar *devno, const gchar
 
 /**
  * bd_s390_zfcp_scsi_offline:
- * @devno a zFCP device number
- * @wwpn a zFCP WWPN identifier
- * @lun a zFCP LUN identifier
- * @error: (out): place to store error (if any)
+ * @devno: a zFCP device number
+ * @wwpn: a zFCP WWPN identifier
+ * @lun: a zFCP LUN identifier
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether a LUN was successfully removed from associated WWPN
  *
@@ -308,7 +283,7 @@ static gboolean  (*_bd_s390_zfcp_offline) (const gchar *devno, const gchar *wwpn
  * @devno: zfcp device number
  * @wwpn: zfcp WWPN (World Wide Port Number)
  * @lun: zfcp LUN (Logical Unit Number)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether a zfcp device was successfully switched offline
  *
@@ -319,100 +294,92 @@ gboolean  bd_s390_zfcp_offline (const gchar *devno, const gchar *wwpn, const gch
 }
 
 
-gpointer load_s390_from_plugin(const gchar *so_name) {
+static gpointer load_s390_from_plugin(const gchar *so_name) {
     void *handle = NULL;
     char *error = NULL;
-    gboolean (*check_fn) (void) = NULL;
     gboolean (*init_fn) (void) = NULL;
 
     handle = dlopen(so_name, RTLD_LAZY);
     if (!handle) {
-        g_warning("failed to load module s390: %s", dlerror());
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load module s390: %s", dlerror());
         return NULL;
     }
 
-    dlerror();
-    * (void**) (&check_fn) = dlsym(handle, "bd_s390_check_deps");
-    if ((error = dlerror()) != NULL)
-        g_debug("failed to load the check() function for s390: %s", error);
-    if (!g_getenv ("LIBBLOCKDEV_SKIP_DEP_CHECKS") && check_fn && !check_fn()) {
-        dlclose(handle);
-        return NULL;
-    }    check_fn = NULL;
-
     dlerror();
     * (void**) (&init_fn) = dlsym(handle, "bd_s390_init");
     if ((error = dlerror()) != NULL)
-        g_debug("failed to load the init() function for s390: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_DEBUG, "failed to load the init() function for s390: %s", error);
+    /* coverity[dead_error_condition] */
     if (init_fn && !init_fn()) {
         dlclose(handle);
         return NULL;
-    }    init_fn = NULL;
+    }
+    init_fn = NULL;
 
     dlerror();
     * (void**) (&_bd_s390_is_tech_avail) = dlsym(handle, "bd_s390_is_tech_avail");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_is_tech_avail: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_is_tech_avail: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_dasd_format) = dlsym(handle, "bd_s390_dasd_format");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_dasd_format: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_dasd_format: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_dasd_needs_format) = dlsym(handle, "bd_s390_dasd_needs_format");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_dasd_needs_format: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_dasd_needs_format: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_dasd_online) = dlsym(handle, "bd_s390_dasd_online");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_dasd_online: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_dasd_online: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_dasd_is_ldl) = dlsym(handle, "bd_s390_dasd_is_ldl");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_dasd_is_ldl: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_dasd_is_ldl: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_dasd_is_fba) = dlsym(handle, "bd_s390_dasd_is_fba");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_dasd_is_fba: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_dasd_is_fba: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_sanitize_dev_input) = dlsym(handle, "bd_s390_sanitize_dev_input");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_sanitize_dev_input: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_sanitize_dev_input: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_zfcp_sanitize_wwpn_input) = dlsym(handle, "bd_s390_zfcp_sanitize_wwpn_input");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_zfcp_sanitize_wwpn_input: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_zfcp_sanitize_wwpn_input: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_zfcp_sanitize_lun_input) = dlsym(handle, "bd_s390_zfcp_sanitize_lun_input");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_zfcp_sanitize_lun_input: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_zfcp_sanitize_lun_input: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_zfcp_online) = dlsym(handle, "bd_s390_zfcp_online");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_zfcp_online: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_zfcp_online: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_zfcp_scsi_offline) = dlsym(handle, "bd_s390_zfcp_scsi_offline");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_zfcp_scsi_offline: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_zfcp_scsi_offline: %s", error);
 
     dlerror();
     * (void**) (&_bd_s390_zfcp_offline) = dlsym(handle, "bd_s390_zfcp_offline");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_s390_zfcp_offline: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_s390_zfcp_offline: %s", error);
 
     return handle;
 }
 
-gboolean unload_s390 (gpointer handle) {
+static gboolean unload_s390 (gpointer handle) {
     char *error = NULL;
     gboolean (*close_fn) (void) = NULL;
 
@@ -432,7 +399,8 @@ gboolean unload_s390 (gpointer handle) {
     dlerror();
     * (void**) (&close_fn) = dlsym(handle, "bd_s390_close");
     if (((error = dlerror()) != NULL) || !close_fn)
-        g_debug("failed to load the close_plugin() function for s390: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_DEBUG, "failed to load the close_plugin() function for s390: %s", error);
+    /* coverity[dead_error_condition] */
     if (close_fn) {
         close_fn();
     }
