@@ -2,25 +2,6 @@ GQuark  bd_swap_error_quark (void) {
         return g_quark_from_static_string ("g-bd-swap-error-quark");
 }
 
-static const gchar* const swap_functions[] = {
-    "bd_swap_is_tech_avail",
-    "bd_swap_mkswap",
-    "bd_swap_swapon",
-    "bd_swap_swapoff",
-    "bd_swap_swapstatus",
-    "bd_swap_set_label",
-    NULL};
-
-const gchar* const* get_swap_functions (void) {
-    return swap_functions;
-}
-
-static const guint8 swap_num_functions = 6;
-
-guint8 get_swap_num_functions (void) {
-    return swap_num_functions;
-}
-
 static gboolean  bd_swap_is_tech_avail_stub (BDSwapTech tech, guint64 mode, GError **error) {
     g_critical ("The function 'bd_swap_is_tech_avail' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
@@ -34,7 +15,7 @@ static gboolean  (*_bd_swap_is_tech_avail) (BDSwapTech tech, guint64 mode, GErro
  * bd_swap_is_tech_avail:
  * @tech: the queried tech
  * @mode: a bit mask of queried modes of operation (#BDSwapTechMode) for @tech
- * @error: (out): place to store error (details about why the @tech-@mode combination is not available)
+ * @error: (out) (optional): place to store error (details about why the @tech-@mode combination is not available)
  *
  * Returns: whether the @tech-@mode combination is available -- supported by the
  *          plugin implementation and having all the runtime dependencies available
@@ -44,29 +25,30 @@ gboolean  bd_swap_is_tech_avail (BDSwapTech tech, guint64 mode, GError **error)
 }
 
 
-static gboolean  bd_swap_mkswap_stub (const gchar *device, const gchar *label, const BDExtraArg **extra, GError **error) {
+static gboolean  bd_swap_mkswap_stub (const gchar *device, const gchar *label, const gchar *uuid, const BDExtraArg **extra, GError **error) {
     g_critical ("The function 'bd_swap_mkswap' called, but not implemented!");
     g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
                 "The function 'bd_swap_mkswap' called, but not implemented!");
     return FALSE;
 }
 
-static gboolean  (*_bd_swap_mkswap) (const gchar *device, const gchar *label, const BDExtraArg **extra, GError **error) = bd_swap_mkswap_stub;
+static gboolean  (*_bd_swap_mkswap) (const gchar *device, const gchar *label, const gchar *uuid, const BDExtraArg **extra, GError **error) = bd_swap_mkswap_stub;
 
 /**
  * bd_swap_mkswap:
  * @device: a device to create swap space on
- * @label: (allow-none): a label for the swap space device
- * @extra: (allow-none) (array zero-terminated=1): extra options for the creation (right now
+ * @label: (nullable): a label for the swap space device
+ * @uuid: (nullable): UUID for the swap space device
+ * @extra: (nullable) (array zero-terminated=1): extra options for the creation (right now
  *                                                 passed to the 'mkswap' utility)
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the swap space was successfully created or not
  *
  * Tech category: %BD_SWAP_TECH_SWAP-%BD_SWAP_TECH_MODE_CREATE
  */
-gboolean  bd_swap_mkswap (const gchar *device, const gchar *label, const BDExtraArg **extra, GError **error) {
-    return _bd_swap_mkswap (device, label, extra, error);
+gboolean  bd_swap_mkswap (const gchar *device, const gchar *label, const gchar *uuid, const BDExtraArg **extra, GError **error) {
+    return _bd_swap_mkswap (device, label, uuid, extra, error);
 }
 
 
@@ -83,7 +65,7 @@ static gboolean  (*_bd_swap_swapon) (const gchar *device, gint priority, GError
  * bd_swap_swapon:
  * @device: swap device to activate
  * @priority: priority of the activated device or -1 to use the default
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the swap device was successfully activated or not
  *
@@ -106,7 +88,7 @@ static gboolean  (*_bd_swap_swapoff) (const gchar *device, GError **error) = bd_
 /**
  * bd_swap_swapoff:
  * @device: swap device to deactivate
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the swap device was successfully deactivated or not
  *
@@ -129,7 +111,7 @@ static gboolean  (*_bd_swap_swapstatus) (const gchar *device, GError **error) =
 /**
  * bd_swap_swapstatus:
  * @device: swap device to get status of
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: %TRUE if the swap device is active, %FALSE if not active or failed
  * to determine (@error) is set not a non-NULL value in such case)
@@ -154,7 +136,7 @@ static gboolean  (*_bd_swap_set_label) (const gchar *device, const gchar *label,
  * bd_swap_set_label:
  * @device: a device to set label on
  * @label: label that will be set
- * @error: (out): place to store error (if any)
+ * @error: (out) (optional): place to store error (if any)
  *
  * Returns: whether the label was successfully set or not
  *
@@ -165,70 +147,91 @@ gboolean  bd_swap_set_label (const gchar *device, const gchar *label, GError **e
 }
 
 
-gpointer load_swap_from_plugin(const gchar *so_name) {
+static gboolean  bd_swap_set_uuid_stub (const gchar *device, const gchar *uuid, GError **error) {
+    g_critical ("The function 'bd_swap_set_uuid' called, but not implemented!");
+    g_set_error (error, BD_INIT_ERROR, BD_INIT_ERROR_NOT_IMPLEMENTED,
+                "The function 'bd_swap_set_uuid' called, but not implemented!");
+    return FALSE;
+}
+
+static gboolean  (*_bd_swap_set_uuid) (const gchar *device, const gchar *uuid, GError **error) = bd_swap_set_uuid_stub;
+
+/**
+ * bd_swap_set_uuid:
+ * @device: a device to set UUID on
+ * @uuid: UUID that will be set
+ * @error: (out) (optional): place to store error (if any)
+ *
+ * Returns: whether the UUID was successfully set or not
+ *
+ * Tech category: %BD_SWAP_TECH_SWAP-%BD_SWAP_TECH_MODE_SET_UUID
+ */
+gboolean  bd_swap_set_uuid (const gchar *device, const gchar *uuid, GError **error) {
+    return _bd_swap_set_uuid (device, uuid, error);
+}
+
+
+static gpointer load_swap_from_plugin(const gchar *so_name) {
     void *handle = NULL;
     char *error = NULL;
-    gboolean (*check_fn) (void) = NULL;
     gboolean (*init_fn) (void) = NULL;
 
     handle = dlopen(so_name, RTLD_LAZY);
     if (!handle) {
-        g_warning("failed to load module swap: %s", dlerror());
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load module swap: %s", dlerror());
         return NULL;
     }
 
-    dlerror();
-    * (void**) (&check_fn) = dlsym(handle, "bd_swap_check_deps");
-    if ((error = dlerror()) != NULL)
-        g_debug("failed to load the check() function for swap: %s", error);
-    if (!g_getenv ("LIBBLOCKDEV_SKIP_DEP_CHECKS") && check_fn && !check_fn()) {
-        dlclose(handle);
-        return NULL;
-    }    check_fn = NULL;
-
     dlerror();
     * (void**) (&init_fn) = dlsym(handle, "bd_swap_init");
     if ((error = dlerror()) != NULL)
-        g_debug("failed to load the init() function for swap: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_DEBUG, "failed to load the init() function for swap: %s", error);
+    /* coverity[dead_error_condition] */
     if (init_fn && !init_fn()) {
         dlclose(handle);
         return NULL;
-    }    init_fn = NULL;
+    }
+    init_fn = NULL;
 
     dlerror();
     * (void**) (&_bd_swap_is_tech_avail) = dlsym(handle, "bd_swap_is_tech_avail");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_swap_is_tech_avail: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_swap_is_tech_avail: %s", error);
 
     dlerror();
     * (void**) (&_bd_swap_mkswap) = dlsym(handle, "bd_swap_mkswap");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_swap_mkswap: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_swap_mkswap: %s", error);
 
     dlerror();
     * (void**) (&_bd_swap_swapon) = dlsym(handle, "bd_swap_swapon");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_swap_swapon: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_swap_swapon: %s", error);
 
     dlerror();
     * (void**) (&_bd_swap_swapoff) = dlsym(handle, "bd_swap_swapoff");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_swap_swapoff: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_swap_swapoff: %s", error);
 
     dlerror();
     * (void**) (&_bd_swap_swapstatus) = dlsym(handle, "bd_swap_swapstatus");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_swap_swapstatus: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_swap_swapstatus: %s", error);
 
     dlerror();
     * (void**) (&_bd_swap_set_label) = dlsym(handle, "bd_swap_set_label");
     if ((error = dlerror()) != NULL)
-        g_warning("failed to load bd_swap_set_label: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_swap_set_label: %s", error);
+
+    dlerror();
+    * (void**) (&_bd_swap_set_uuid) = dlsym(handle, "bd_swap_set_uuid");
+    if ((error = dlerror()) != NULL)
+        bd_utils_log_format (BD_UTILS_LOG_WARNING, "failed to load bd_swap_set_uuid: %s", error);
 
     return handle;
 }
 
-gboolean unload_swap (gpointer handle) {
+static gboolean unload_swap (gpointer handle) {
     char *error = NULL;
     gboolean (*close_fn) (void) = NULL;
 
@@ -238,11 +241,13 @@ gboolean unload_swap (gpointer handle) {
     _bd_swap_swapoff = bd_swap_swapoff_stub;
     _bd_swap_swapstatus = bd_swap_swapstatus_stub;
     _bd_swap_set_label = bd_swap_set_label_stub;
+    _bd_swap_set_uuid = bd_swap_set_uuid_stub;
 
     dlerror();
     * (void**) (&close_fn) = dlsym(handle, "bd_swap_close");
     if (((error = dlerror()) != NULL) || !close_fn)
-        g_debug("failed to load the close_plugin() function for swap: %s", error);
+        bd_utils_log_format (BD_UTILS_LOG_DEBUG, "failed to load the close_plugin() function for swap: %s", error);
+    /* coverity[dead_error_condition] */
     if (close_fn) {
         close_fn();
     }
